from apiclient import APIClient
from click_help_colors import HelpColorsGroup, HelpColorsCommand
from importlib import util
from IPython.display import display
from jsondiff import JsonDiffer
from os import path
from pydoc import Helper
from tabulate import tabulate
import click
import json
import logging
import os
import pandas as pd
import requests
import sys
import warnings



# Set defaults
pd.set_option('display.max_rows', None)
pd.set_option('display.max_columns', 4)
pd.set_option('display.width', 100)
pd.set_option('display.colheader_justify', 'center')
pd.set_option('display.precision', 3)
requests.packages.urllib3.disable_warnings()
warnings.simplefilter(action='ignore', category=FutureWarning)
CONTEXT_SETTINGS = dict(auto_envvar_prefix="PC")

class Environment:
    def __init__(self):
        self.verbose = False
        self.home = os.getcwd()

    def log(self, msg, *args):
        """Logs a message to stderr."""
        if args:
            msg %= args
        click.echo(msg, file=sys.stderr)

    def vlog(self, msg, *args):
        """Logs a message to stderr only if verbose is enabled."""
        if self.verbose:
            self.log(msg, *args)


pass_environment = click.make_pass_decorator(Environment, ensure=True)
cwpp_folder = os.path.abspath(os.path.join(os.path.dirname(__file__), "cwpp"))
cspm_folder = os.path.abspath(os.path.join(os.path.dirname(__file__), "cspm"))


class PrismaCloudCLI(click.MultiCommand):
    def list_commands(self, ctx):
        rv = []

        # Iterate through cwpp commands
        for filename in os.listdir(cwpp_folder):
            if filename.endswith(".py") and filename.startswith("cmd_"):
                rv.append(filename[4:-3])

        # Iterate through cspm commands
        for filename in os.listdir(cspm_folder):
            if filename.endswith(".py") and filename.startswith("cmd_"):
                rv.append(filename[4:-3])

        rv.sort()
        return rv

    def get_command(self, ctx, name):
        try:
            mod = __import__(f"cwpp.cmd_{name}", None, None, ["cli"])
        except ImportError as e:
            # Importing cwpp command did not work. Try to import cspm
            try:
                mod = __import__(f"cspm.cmd_{name}", None, None, ["cli"])
            except ImportError as e:
                return
        return mod.cli


@click.command(cls=PrismaCloudCLI, context_settings=CONTEXT_SETTINGS)
@click.option("-v", "--verbose", is_flag=True, help="Enables verbose mode.")
@click.option("-o", "--output", type=click.Choice(['text', 'csv', 'json', 'html', 'columns']), default='text')
@click.option("-c", "--config", 'configuration', help="Select configuration ~/.prismacloud/[CONFIGURATION].json", default="credentials")
@pass_environment
def cli(ctx, verbose, configuration, output):
    """A PrismaCloud command line interface."""
    ctx.verbose = verbose
    ctx.configuration = configuration
    ctx.output = output

    if ctx.verbose:
        logging.basicConfig(
         level=logging.DEBUG, format='%(asctime)s - %(levelname)s - [%(filename)s:%(lineno)d] - %(message)s')
    else:
        logging.basicConfig(
         level=logging.ERROR, format='%(asctime)s - %(levelname)s - [%(filename)s:%(lineno)d] - %(message)s')

def output(data):
    # Retrieve parameters
    params = click.get_current_context().find_root().params


    # We have data from our request, send to dataframe
    try:
        df = pd.json_normalize(data)

        # Do some optimization on our dataframe
        try:
            df['time'] = pd.to_datetime(df.time)
            df.fillna('', inplace=True)
        except:
            logging.debug('No time field')
        
        # We have a dataframe, output here
        if params["output"] == "text":
            click.echo(tabulate(df, headers = 'keys', tablefmt = 'psql'))
        if params["output"] == "json":
            click.echo(df.to_json())
        if params["output"] == "csv":
            click.echo(df.to_csv())
        if params["output"] == "html":
            click.echo(df.to_html())
        if params["output"] == "columns":
            for column in df.columns:
                click.echo(column)
    except Exception as e:
        # There is no dataframe, might be just a single value
        # like version. Show data
        click.echo(data)

        logging.debug("Error ingesting data into dataframe: {}".format(e))


if __name__ == "__main__":
    #cli()

    try:
        cli()
    except Exception as e:
        logging.error("An error has occured: {}".format(e))




import logging
import os
import json
import click
import prismacloud.version
from apiclient import *

class PrismaCloudClient(APIClient):
    def request(self, method):
        url = "https://{}/api/v1/{}".format(PCC_API_ENDPOINT, method)
        return self.get(url)


def get(method):
    click.get_current_context()

    # Before sending out the request, we are going to retrieve the configuration
    getConfig()

    logging.debug("Calling api method: /{}".format(method))
    logging.debug("Username: {}".format(ACCESS_KEY_ID))
    client = PrismaCloudClient(
        authentication_method=BasicAuthentication(username=ACCESS_KEY_ID, password=SECRET_KEY),
        response_handler=JsonResponseHandler
    )
    
    return client.request(method)


def getParamFromJson(config_file):
    logging.debug('Retrieving configuration')
    f = open(config_file,)
    params = json.load(f)
    try:
        api_endpoint = params["api_endpoint"]
        pcc_api_endpoint = params["pcc_api_endpoint"]
        access_key_id = params["access_key_id"]
        secret_key = params["secret_key"]
        # Closing file
        f.close()
        return api_endpoint, pcc_api_endpoint, access_key_id, secret_key
    except:
        api_endpoint = False
        pcc_api_endpoint = params["pcc_api_endpoint"]
        access_key_id = params["access_key_id"]
        secret_key = params["secret_key"]
        # Closing file
        f.close()
        return api_endpoint, pcc_api_endpoint, access_key_id, secret_key


def getConfig():
    global params
    params = click.get_current_context().find_root().params

    global API_ENDPOINT, PCC_API_ENDPOINT, ACCESS_KEY_ID, SECRET_KEY, token, version
    version = prismacloud.version.version

    logging.info('Running prismacloud-cli version {}'.format(version))
    PRISMA_CLOUD_DIRECTORY = os.environ['HOME'] + "/.prismacloud/"

    if os.path.exists(PRISMA_CLOUD_DIRECTORY):
        if os.path.exists(os.environ['HOME'] + "/.prismacloud/{}.json".format(params['configuration'])):
            try:
                CONFIG_FILE = PRISMA_CLOUD_DIRECTORY + \
                    "{}.json".format(params['configuration'])
                API_ENDPOINT, PCC_API_ENDPOINT, ACCESS_KEY_ID, SECRET_KEY = getParamFromJson(
                    CONFIG_FILE)
            except Exception as e:
                logging.info(e)
        else:
            CONFIG_FILE = os.environ['HOME'] + "/.prismacloud/credentials.json"
            API_ENDPOINT, PCC_API_ENDPOINT, ACCESS_KEY_ID, SECRET_KEY = getParamFromJson(
                CONFIG_FILE)

        logging.debug("Config loaded: {}".format(params['configuration']))

    else:
        logging.info(
            'Prisma cloud directory does not exists, let\'s create one in your $HOME/.prismacloud')
        os.makedirs(PRISMA_CLOUD_DIRECTORY)
        CONFIG_FILE = PRISMA_CLOUD_DIRECTORY + "credentials.json"
        API_ENDPOINT = input(
            "Enter CSPM API Endpoint (OPTIONAL if PCCE), eg: api.prismacloud.io: ")
        PCC_API_ENDPOINT = input(
            "Enter CWPP API Endpoint, eg: us-east1.cloud.twistlock.com/<tenant-id>: ")
        ACCESS_KEY_ID = input("Enter the access key ID: ")
        SECRET_KEY = input("Enter the secret key: ")
        API_FILE = {
            "api_endpoint": API_ENDPOINT,
            "pcc_api_endpoint": PCC_API_ENDPOINT,
            "access_key_id": ACCESS_KEY_ID,
            "secret_key": SECRET_KEY
        }

        json_string = json.dumps(API_FILE, sort_keys=True, indent=4)

        with open(CONFIG_FILE, 'w') as outfile:
            outfile.write(json_string)

    return



import os
from setuptools import setup
import logging
from importlib import util
from os import path

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

spec = util.spec_from_file_location(
    "prismacloud_cli.version", os.path.join("prismacloud", "version.py")
)

# noinspection PyUnresolvedReferences
mod = util.module_from_spec(spec)
spec.loader.exec_module(mod)  # type: ignore
version = mod.version  # type: ignore

setup(
    extras_require={},
    install_requires=[
        "requests",
        "pandas",
        "jsondiff",
        "importlib",
        "ipython",
        "click",
        "click_help_colors",
        "tabulate"
    ],
    name = "prismacloud-cli",
    version = version,
    python_requires=">=3.7",
    author = "Steven de Boer",
    author_email = "stdeboer@paloaltonetworks.com",
    maintainer = "Simon Melotte",
    maintainer_email = "smelotte@paloaltonetworks.com",
    description = ("Prisma Cloud CLI"),
    license = "BSD",
    keywords = "prisma cloud cli",
    url = "https://github.com/se-cloud-emea/prismacloud-cli",
    packages=['prismacloud'],
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Programming Language :: Python",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License"
    ],
    entry_points="""
        [console_scripts]
        pc=prismacloud.cli:cli
    """,
)